# Copyright (C) 1998  Eleftherios Gkioulekas <lf@amath.washington.edu>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

# =============================
# == Usage related functions ==
# =============================

function usage
{
 cat << EOF
Creates directory tree for new software package.
Usage: acmkdir [OPTIONS] <directoryname>

* Commandline flags:
   --help           This message
   --version        Version information
   -t, --type=TYPE  Make top-level directory of type TYPE
   -doc             Make a texidoc subdirectory
   -latex           Make a latex   subdirectory

* Types of toplevel directories supported (-t --type)
   default     -> Language C++, uses the LF macros
   traditional -> Language C, uses traditional autoconf macros
   fortran     -> Language C++ with Fortran
   f77         -> not implemented yet
   f90         -> not implemented yet

Bug reports: lf@amath.washington.edu
EOF
 exit
}

function version
{
 cat << EOF
automkdir $VERSION - Generate directory tree for new software package
Copyright (C) 1997 Eleftherios Gkioulekas <lf@amath.washington.edu>
This is free software, and you are welcome to redistribute it and modify it 
under certain conditions. There is ABSOLUTELY NO WARRANTY for this software.
For legal details see the GNU General Public License.
EOF
 exit
}

function invalid_usage
{
 cat <<EOF
Invalid usage. For usage information type:
% acmkdir --help
EOF
 exit
}

# =========================================
# == Make the standard information files ==
# =========================================

# ---------------------------------------------------
# This function creates a template NEWS file in the 
# present working directory.
# ---------------------------------------------------

function make_NEWS
{
 cat >> NEWS <<EOF
${package_name} -- History of visible changes.

Copyright (C) $this_year, $your_name <${email_address}>
See the end for copying conditions.

Please send ${package_name} bug reports to ${email_address}.

Version 0.1

* List your features here

-------------------------------------------------------
Copying information:

Copyright (C) $this_year, $your_name <${email_address}>

   Permission is granted to anyone to make or distribute verbatim copies
   of this document as received, in any medium, provided that the
   copyright notice and this permission notice are preserved,
   thus giving the recipient permission to redistribute in turn.

   Permission is granted to distribute modified versions
   of this document, or of portions of it,
   under the above conditions, provided also that they
   carry prominent notices stating who last changed them.

EOF
}

# ----------------------------------------------------
# This function creates a template README file in the
# present working directory
# ----------------------------------------------------

function make_README
{
 cat >> README <<EOF
Welcome to ${package_name}.

${package_name} is free software. Please see the file COPYING for details.
For documentation, please see the files in the doc subdirectory.
For building and installation instructions please see the INSTALL file.
EOF
}

# -------------------------------------------------------
# This function creates a template AUTHORS file in the
# present working directory
# -------------------------------------------------------

function make_AUTHORS
{
 cat >> AUTHORS <<EOF
Authors of ${package_name}.
See also the files THANKS and ChangeLog.

${your_name} designed and implemented ${package_name}.
EOF
}

# --------------------------------------------------------
# This function creates the template THANKS file in the
# present working directory
# --------------------------------------------------------

function make_THANKS
{
 cat >> THANKS <<EOF
${package_name} THANKS file

${package_name} has originally been written by ${your_name}.
Many people have further contributed to FOO by reporting problems, 
suggesting various improvements, or submitting actual code. Here is
a list of these people. Help me keep it complete and exempt of errors.
EOF
}

# ---------------------------------------------------------
# This function creates an alternative INSTALL file in the
# present working directory
# ---------------------------------------------------------

function make_INSTALL_nonstandard
{
 rm -f INSTALL
 ln -s $PREFIX/share/autotools/new_install.txt INSTALL
}

# ---------------------------------------------------------------
# This function creates a Makefile for an m4 subdirectory
# and provides also a copy of mkm4.pl in case the user wants it
# FIXME: Do I want a better way of dealing with this?
# ---------------------------------------------------------------

function make_m4_makefile
{
 cat >> m4/Makefile.am <<EOF
# Install m4 macros in this directory
m4datadir = \$(datadir)/aclocal

# List your m4 macros here
m4macros = 

# The following is boilerplate
m4data_DATA = \$(m4macros) 
EXTRA_DIST = \$(m4data_DATA) 

EOF
}

# ==============================================
# == Routines for making mode dependent filei ==
# ==============================================

# --------------------------------------------------------------------
# These functions make the default configure.in and Makefile.am files
# The default configure.in file assumes that you are programming
# only in C++ and uses the LF macros instead of the standard
# Autoconf macros for configuring the compiler.
# It also includes a call to LF_CPP_PORTABILITY
# --------------------------------------------------------------------

function make_toplevel_configure_default
{
 cat >> configure.in <<EOF
AC_INIT(reconf)
AM_CONFIG_HEADER(config.h)
AM_INIT_AUTOMAKE(${dirname},0.1)
LF_CONFIGURE_CC
LF_CONFIGURE_CXX
LF_HOST_TYPE
LF_SET_WARNINGS
AC_PROG_RANLIB
AC_OUTPUT(Makefile doc/Makefile m4/Makefile src/Makefile)
EOF
}

function make_toplevel_makefile_default
{
 cat >> Makefile.am <<EOF
EXTRA_DIST = reconf configure
SUBDIRS = m4 doc src
EOF
}

function make_reconf_default
{
 cat >> reconf <<EOF
#!/bin/sh
rm -f config.cache
rm -f acconfig.h
echo "- aclocal."
aclocal -I m4
echo "- autoconf."
autoconf
echo "- acconfig."
acconfig
echo "- autoheader."
autoheader
echo "- automake."
automake -a
exit
EOF
 chmod u+x reconf
}

function make_local_default
{
 return
}

# --------------------------------------------------------------------
# These functions make the traditional configure.in and Makefile.am
# files. This is for people who want to follow the GNU coding
# standards a lot more closer than I do. It assumes that you are
# using C only and does not use any LF macros
# --------------------------------------------------------------------

function make_toplevel_configure_traditional
{
 cat >> configure.in <<EOF
AC_INIT(reconf)
AM_INIT_AUTOMAKE(${dirname},0.1)
AC_PROG_CC
AC_AIX
AC_ISC_POSIX
AC_MINIX
AC_STDC_HEADERS
AC_PROG_RANLIB
AC_OUTPUT(Makefile doc/Makefile m4/Makefile src/Makefile)
EOF
}

function make_toplevel_makefile_traditional
{
 make_toplevel_makefile_default
}

function make_reconf_traditional
{
 cat >> reconf <<EOF
#!/bin/sh
rm -f config.cache
echo "- aclocal."
aclocal -I m4
echo "- autoconf."
autoconf
echo "- autoheader."
autoheader
echo "- automake."
automake -a
exit
EOF
 chmod u+x reconf
}

function make_local_traditional
{
 gpl -cS acconfig.h
 cat >> acconfig.h <<EOF

/* These are required by Automake */
#undef PACKAGE
#undef VERSION
EOF
}

# ---------------------------------------------------------------------
# These functions make configure.in and Makefile.am files appropriate
# for packages that use mixed C++ and Fortran.
# ---------------------------------------------------------------------

function make_toplevel_configure_fortran
{
 cat >> configure.in <<EOF
AC_INIT(reconf)
AM_CONFIG_HEADER(config.h)
AM_INIT_AUTOMAKE(${dirname},0.1)
LF_CONFIGURE_CC
LF_CONFIGURE_CXX
AC_PROG_RANLIB
LF_HOST_TYPE
LF_PROG_F77_PREFER_F2C_COMPATIBILITY
dnl LF_PROG_F77_PREFER_NATIVE_VERSION
LF_PROG_F77
LF_SET_WARNINGS
AC_CONFIG_SUBDIRS(fortran/f2c fortran/libf2c)
AC_OUTPUT([Makefile fortran/Makefile f2c_comp
          doc/Makefile m4/Makefile src/Makefile ])
EOF
}

function make_toplevel_makefile_fortran
{
 cat >> Makefile.am <<EOF
EXTRA_DIST = reconf configure
SUBDIRS = fortran m4 doc src
EOF
}

function make_reconf_fortran
{
 make_reconf_default
}

function make_local_fortran
{
 echo "+ Installing f2c."
 mkfortran > /dev/null
 cp $PREFIX/share/autotools/f2c_comp.sh f2c_comp.in
}

# ============================================================
# == These are the highlevel functions for creating various ==
# == types of directories                                   ==
# ============================================================

# -------------------------------------------------------
# This function creates the toplevel package directory.
# FIXME: Add more documentation here
# -------------------------------------------------------

function create_toplevel_directory 
{
 # Greet the user
 echo "Ready to create a new distribution skeleton directory."
 echo "The current working directory is: "
 echo " --> `pwd`"
 echo "If you make a mistake and need to abort, press ctrl-C."
 echo " "

 # Get the parameters
 echo -n "Name of distribution: "
 read package_name
 echo -n "Your full name:       "
 read your_name
 echo -n "Your email address:   "
 read email_address

 # Check whether to go ahead.
 echo " "
 echo -n "Do you want to proceed? (y/n) "
 local yesno
 read yesno
 if test "$yesno" != "y"
 then
   echo "The force is with you young Skywalker,"
   echo "but you are not a Jedi yet."
   exit
 fi

 # Get the today's year
 this_year=`date +%Y`

 # Make the directories
 echo "+ Mode is ${mode}"
 echo "+ Making directory ${dirname}"
 mkdir ${dirname}-0.1
 cd ${dirname}-0.1
 mkdir src doc m4

 # Put together copyright templates for the following
 # source files
 gpl -l COPYING
 gpl -am Makefile.am
 gpl -am doc/Makefile.am
 gpl -am src/Makefile.am
 gpl -am m4/Makefile.am
 gpl -shS configure.in
 gpl -shS reconf
 chmod u+x reconf

 # Create templates for the standard information files
 touch NEWS README AUTHORS THANKS INSTALL ChangeLog
 make_NEWS
 make_README
 make_AUTHORS
 make_THANKS
 make_INSTALL_nonstandard
 make_m4_makefile

 # Invoke the mode-dependent functions to create the toplevel
 # configure.in and Makefile.am and the reconf script
 make_toplevel_configure_${mode}
 make_toplevel_makefile_${mode}
 make_reconf_${mode}
 make_local_${mode}

 # Run the reconf script
 echo "+ Running reconf"
 sh ./reconf 

 # Important message
 cat <<EOF 

Distribution directory is ready.
Please make sure to keep the files AUTHORS, NEWS, README, THANKS up to
date before cutting a distribution.
EOF
}

# ----------------------------------------------------------------
# This function creates a subdirectory for writing texinfo 
# documentation that is preproccessed with texidoc. It uses the 
# doc_makefile.am file as standard makefile.
# $dirname contains the name of the directory to create
# ----------------------------------------------------------------

function create_doc_directory
{
 echo "+ Making directory ${dirname}"
 mkdir ${dirname}
 cd ${dirname}
 cp $PREFIX/share/autotools/doc_makefile.am Makefile.am
 cp `which texi2html` texi2html.pl
 cp `which texidoc`   texidoc.pl
 cp `which texirm`    texirm.sh
 cat <<EOF

Documentation subdirectory is ready.
Please make sure to add a reference to it, to the appropriate Makefile.am
and to add an entry for it in configure.in.
You will also need to customize the template Makefile.am in this directory.
EOF
}

# ------------------------------------------------------------------
# This function creates a subdirectory for writing latex based 
# documentation. It uses latex_makefile.am as the standard
# makefile. $dirname contains the name of the directory to create
# ------------------------------------------------------------------

function create_latex_directory
{
 echo "Latex subdirectories are currently unsupported."
 exit
}

############################################################################
############################################################################

# 
# Parse the command-line arguments
#

# Force some command-line arguments to be passed
if test $# -eq 0
then
  invalid_usage
fi

# Check for --help and --version
if test $1 = --help
then
  usage
fi
if test $1 = --version
then
  version
fi

# FIXME:
# Check for -m4 , -texi , -latex here
# and take different action

if test $1 = -m4
then
  shift
  dirname=$1
  create_m4_directory
  exit
fi

if test $1 = -doc
then
  shift
  dirname=$1
  create_doc_directory
  exit
fi

if test $1 = -latex
then
  shift
  dirname=$1
  create_latex_directory
  exit
fi

# Set the mode
mode="default"
if test $1 = -t || test $1 = --type
then
  shift
  mode=$1
  shift
fi

# Check against allowed modes
allowed_modes="default traditional fortran"
mode_is_ok=0
for i in ${allowed_modes}
do
  if test $i = $mode
  then
    mode_is_ok=1
  fi
done
if test $mode_is_ok = 0
then
  invalid_usage
fi

# Go ahead and create the toplevel directory
dirname=$1
if test -z $dirname
then
  invalid_usage
fi
create_toplevel_directory
exit
