# The following script makes it easy for you to make your own Slackware
# packages out of various installations that you do
#
# Copyright (C) 1996  Eleftherios Gkioulekas <lf@amath.washington.edu>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#                    =============================
#                    == Usage related functions ==
#                    =============================

function usage
{
 cat << EOF
To create a package do the following:

* Before running "make install" make a list of the directory tree under
  which your software will be installed. That's probably /usr/local but you
  might need to do /usr alltogether depending on what you install.
    % buildpkg -l /usr > list1

* Now install your software. After you are done make a list of the same 
  directory tree:
    % buildpkg -l /usr > list2

* Now you can create the package. To preview:
    % buildpkg -m list1 list2 > packagelist

* To actually build the package:
    % buildpkg -b packagelist packagename

A file packagename.tgz will be created. You are done!
This program is protected by copyright law. For copyright information type
    % buildpkg -c
Enjoy!
EOF
}

function copyright
{
 cat << EOF
Copyright (C) 1997 Eleftherios Gkioulekas <lf@amath.washington.edu>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
EOF
}


#              ======================================
#              == Perl scripts doing the hard work ==
#              ======================================

# -----------------------------------------------------------------------
# The following function lists all the files under a directory in
# full pathname form. Subdirectories are not shown, but the files
# under the subdirectories are.
#
# Usage:
# list_files <directoryname>
# -----------------------------------------------------------------------

function list_files
{
 perl -e '
   do listdir($ARGV[0]);

   sub listdir
   {
    local($dir) = @_;
    local($dev,$ino,$mode,$subcount);

    opendir(DIR,$dir) || die "Can not open $dir";
    local(@filenames) = readdir(DIR);
    close(DIR);

    for (@filenames)
    {
     next if $_ eq ".";
     next if $_ eq "..";
     $name = "$dir/$_";
     if (-d $name && !( -l $name ))
     {
      do listdir($name);
     }
    else
    {
     print $name,"\n";
    }
   }
  } ' $1
}

# -----------------------------------------------------------------------
# Suppose you have two files, one listing the files on your hard drive 
# before an installation and one listing the files after installation.
# This function will load these two files in and list the files that were
# added during installation. 
#
# Usage:
# list_newfiles <before_list> <after_list>
# -----------------------------------------------------------------------

function list_newfiles
{
 perl -e '
   $before_list = $ARGV[0];
   $after_list  = $ARGV[1];
   open(FILE,$before_list);
   @files_before = <FILE>;
   close(FILE);
   open(FILE,$after_list);
   @files_after = <FILE>;
   close(FILE);
   local(%mark);
   foreach $file (@files_before) { $mark{$file}++; }
   foreach $file (@files_after)  
   { print $file unless $mark{$file}; }
  ' $1 $2
}

# -----------------------------------------------------------------------
# The following function will make a package from a list
#
# Usage:
# make_package_from_list <list> <packagename>
# -----------------------------------------------------------------------

function make_package_from_list
{
 local filelist=$1
 local packagename=$2
 tar cf  ${packagename}.tar
 echo "Building package. This may take a while"
 cat ${filelist} | awk -v tarfile=${packagename}.tar '
  { print "tar rf " tarfile " " $1 }' | bash 2> /dev/null
 echo "Compressing package"
 gzip -9 ${packagename}.tar
 mv ${packagename}.tar.gz ${packagename}.tgz
}

#                =========================================
#                == Here is the main part of the script ==
#                =========================================

# Grab the flag and branch accoringly
if test $# -ne 0
then
  FLAG=$1
  shift
  case $FLAG in
  -c)
    copyright
    ;;
  -l)
    if test $# -eq 1
    then
      list_files $1
    else
      usage
    fi
    ;;
  -m)
    if test $# -eq 2
    then 
      list_newfiles $1 $2
    else
      usage
    fi
    ;;
  -b)
    if test $# -eq 2
    then
      make_package_from_list $1 $2
    else
      usage
    fi
    ;;
  *)
    usage
    ;;
  esac
fi
