# Copyright (C) 1988 Eleftherios Gkioulekas <lf@amath.washington.edu>
#  
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
# =============================
# == Usage related functions ==
# =============================

function usage
{
 cat << EOF
To add an entry to ChangeLog type:
  % changelog
This works even if the current directory is not the one where the ChangeLog
is located. Follow the prompts.

Once you are done doing the changes of the day do:
  % changelog -sync
Until then changes are stored in ChangeLog.tmp. This command will add these
changes to the main ChangeLog
 
To view recent changes (in ChangeLog.tmp):
  % changelog -view
EOF
}

function version
{
 cat << EOF
changelog $VERSION - The ChangeLog manager.
Copyright (C) 1998 Eleftherios Gkioulekas <lf@amath.washington.edu>
This is free software, and you are welcome to redistribute it and modify it 
under certain conditions. There is ABSOLUTELY NO WARRANTY for this software.
For legal details refer to Version 2 of the GNU General Public License.
EOF
}

# ========================================================
# == Locate the directory where ChangeLog is located at ==
# ========================================================

function find_changelog
{
 while true
 do
   # If we found it return
   if test -r ChangeLog
   then
     echo "ChangeLog at " $(pwd)
     return
   fi

   # If we will never find it, complain
   if test `pwd` = "/"
   then
     echo "ERROR: Can't locate ChangeLog!"
     exit
   fi

   # Otherwise, go up one directory
   cd ..
 done
}

# ======================================
# == Add a new entry to the ChangeLog ==
# ======================================

function new_entry
{
 local firstline=1
 local line=""
 echo "ChangeLog entry:"
 echo "(Terminate with a single dot on its own line.)"
 rm -f /tmp/changelog.$$
 touch /tmp/changelog.$$
 while true
 do
   # Read in the line
   echo -n "> "
   read line
   # If this is the last line then we're done
   if test "$line" = "."
   then
     touch ChangeLog.tmp
     mv ChangeLog.tmp _ChangeLog.tmp
     cat /tmp/changelog.$$ >> ChangeLog.tmp
     echo " " >> ChangeLog.tmp
     cat _ChangeLog.tmp >> ChangeLog.tmp
     rm -f _ChangeLog.tmp
     rm -f /tmp/changelog.$$
     return
   fi  
   # Depending on whether this is the first line add it in
   if test $firstline -eq 1
   then
     echo "   * " ${line} >> /tmp/changelog.$$
     firstline=0
   else
     echo "     " ${line} >> /tmp/changelog.$$
   fi
 done
}

# =======================================
# == End the current ChangeLog session ==
# =======================================

function sync_session
{
 local author_name
 local email_address
 echo "Checking in current session to main ChangeLog."
 echo -n "Author name:   "
 read author_name
 echo -n "Email address: "
 read email_address
 mv ChangeLog _ChangeLog
 touch ChangeLog
 echo $(date) >> ChangeLog
 echo "${author_name} <${email_address}>" >> ChangeLog
 echo " " >> ChangeLog
 cat ChangeLog.tmp >> ChangeLog
 echo " " >> ChangeLog
 cat _ChangeLog >> ChangeLog
 rm -f _ChangeLog
 rm -f ChangeLog.tmp
 echo "Done."
}

# =====================================================================
# =====================================================================
 
# If we passed more than one argument then that's
# not correct usage
if test $# -gt 1
then
  cat << EOF
Invalid usage. For more help:
  % changelog --help
EOF
fi

# If we pass no argument, then do standard thing
if test $# -eq 0
then
  find_changelog
  new_entry
  exit
fi

# If we pass one argument, then depends on what the argument is
if test $# -eq 1
then
  if test $1 = --help
  then
    usage
    exit
  fi
  if test $1 = --version
  then
    version
    exit
  fi
  if test $1 = -sync
  then
    find_changelog
    sync_session
    exit
  fi
  if test $1 = -view
  then
    find_changelog
    cat ChangeLog.tmp
    exit
  fi
  cat << EOF
Invalid usage. For more help:
  % changelog --help
EOF
fi

exit
