;;; Copyright (C) 1998 Eleftherios Gkioulekas <lf@amath.washington.edu>
;;;  
;;; This program is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2 of the License, or
;;; (at your option) any later version.
;;;  
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;  
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;;;  

;;
;; Alias texinfo-multiple-files-update to texi-update
;;

(require 'texinfo)
(fset 'texi-update 'texinfo-multiple-files-update)

;;
;; Define commands to automagically handle @example and @end example
;; You should map certain keys to these functions so that you can
;; easily make blocks of examples:
;;   (global-set-key [f9]  'texi-insert-@example)
;;   (global-set-key [f10] 'texi-insert-@smallexample)
;;

(defun texi-insert-@example ()
  "Insert an @example @end example block"
  (interactive)
  (beginning-of-line)
  (insert "@example\n")
  (insert "\n")
  (insert "@end example\n")
  (insert "@noindent\n")
  (previous-line 3)
  (beginning-of-line))

(defun texi-insert-@smallexample ()
  "Insert an @smallexample block"
  (interactive)
  (beginning-of-line)
  (insert "@smallexample\n")
  (insert "\n")
  (insert "@end smallexample\n")
  (insert "@noindent\n")
  (previous-line 3)
  (beginning-of-line))
 
;;
;; The following define the command M-x texi-template which
;; will insert a standard template for a new Texinfo document
;;

(defun texi-get-header-from-user ()
  "This routine prompts the user for information and provides with the"
  "header for the Texinfo template file. The rest of the template is"
  "provided through a defconst"
  (concat "\\input texinfo\n"
          "\n"
          "@c %**start of header\n"
          "@setfilename " (read-string "File name for info file: ") "\n"
          "@set TITLE "   (read-string "Document title: ") "\n"
          "@settitle @value{TITLE}\n"
          "@c %**end of header\n"
	  "\n"
          "@c ---------------------------------------- \n"
          "@c Include configuration information here.\n"
          "@c ---------------------------------------- \n"
          "@set UPDATED " (format-time-string "%Y-%m-%d" (current-time)) "\n"
          "@set EDITION 1\n"
          "@set OWNER " user-full-name "\n"
          "@set THEOWNER " user-full-name "\n"
          "@set YEARS " (format-time-string "%Y" (current-time)) "\n"))

(defconst texi-the-rest-of-the-template
 (concat
 "\n"
"@c ------------------------------------------------------------\n"
"@c Formatting style commands. You may insert more of them here.\n"
"@c For example: headings, indices, etc...\n"
"@c ------------------------------------------------------------\n"
"@setchapternewpage odd\n"
"\n"
"@c ---------------------------------\n"
"@c Canonical title related template\n"
"@c ---------------------------------\n"
"\n"
"@shorttitlepage @value{TITLE}\n"
"@titlepage\n"
"@title @value{TITLE}\n"
"@subtitle Edition @value{EDITION}\n"
"@subtitle @value{UPDATED}\n"
"@author @value{OWNER}\n"
"@page\n"
"@vskip 0pt plus 1filll\n"
"Copyright @copyright{} @value{YEARS} @value{OWNER}. All rights reserved.\n"
"\n"
"Permission is granted to make and distribute verbatim copies of\n"
"this manual provided the copyright notice and this permission notice\n"
"are preserved on all copies.\n"
"\n"
"Permission is granted to copy and distribute modified versions of this\n"
"manual under the conditions for verbatim copying, provided that the entire\n"
"resulting derived work is distributed under the terms of a permission\n"
"notice identical to this one.\n"
"\n"
"Permission is granted to copy and distribute translations of this manual\n"
"into another language, under the above conditions for modified versions,\n"
"except that this permission notice may be stated in a translation approved\n"
"by the Free Software Foundation.\n"
"@end titlepage\n"
"\n"
"@c ---------------------\n"
"@c Menu stuff for info\n"
"@c ----------------------\n"
"\n"
"@ifinfo\n"
"@node Top\n"
"@comment node-name, next, previous, up\n"
"@top @value{TITLE}\n"
"@end ifinfo\n"
"\n"
"@c ===================================================================\n"
"@c ===================================================================\n"
"\n"
"@c Include chapters and indices here\n"
"\n"
"\n"
"@c ===================================================================\n"
"@c ===================================================================\n"
"\n"
"@c @shortcontents\n"
"@contents\n"
"@bye\n"
"\n"))

(defun texi-template ()
  "Insert a standard template file for a new Texinfo document."
  (interactive)
  (insert (texi-get-header-from-user) texi-the-rest-of-the-template))

;;
;; Provide
;;

(provide 'texinfo-xtra)

